@@ -1,7 +1,7 @@
-/* $Id: er_fileutils.c,v 1.33 2011/01/10 15:36:27 kbanse Exp $
+/* $Id: er_fileutils.c,v 1.36 2012/03/14 09:26:56 kbanse Exp $
  *
  *   This file is part of the ESO Common Pipeline Library
- *   Copyright (C) 2001-2006 European Southern Observatory
+ *   Copyright (C) 2001-2012 European Southern Observatory
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -20,9 +20,9 @@
 
 /*
  * $Author: kbanse $
- * $Date: 2011/01/10 15:36:27 $
- * $Revision: 1.33 $
- * $Name: esorex-3_9_0 $
+ * $Date: 2012/03/14 09:26:56 $
+ * $Revision: 1.36 $
+ * $Name: esorex-3_9_6 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -58,7 +58,6 @@
 #include "er_plugin.h"
 #include "er_fileutils.h"
 
-#define PATHSET_MAX     1024
 #define DEV_BLOCKSIZE   4096
 
 #define Xspace(x)  (x == ' ') || (x == '\t') || (x == '\n')
@@ -85,7 +84,7 @@ static char trace_msg[80];
  *
  * @param  path      Input path
  *
- * @return A string with the replaced varaibles.
+ * @return A string with the replaced variables.
  *
  * This function performs the replacement of any environment 
  * variables. It only works if the WORDEXP package is available 
@@ -163,6 +162,12 @@ return expanded_path;
  *
  * This function also performs the replacement of any environment 
  * variables.
+ *
+ * OJO: this routine returns a pointer to a static string,
+ *      if you need the returned filename later on, you must copy it
+ *      to a string of PATHSET_MAX length
+ *      otherwise the returned pointer may point later on to something
+ *      very different   :-(
  */
 /**********************************************************************/
 
@@ -172,7 +177,7 @@ const char *er_fileutils_tilde_replace (const char * name)
 static char str[PATHSET_MAX];
 char *marker, *cpp;
 
-int  len;
+int  len, lname;
 
 
 
@@ -181,18 +186,20 @@ ER_TRACE(__func__)
 
 if (name == NULL) return NULL; 
 
+lname = (int) strlen(name);
 
 /* Only do this, if we start with a tilde (~) */
 
 if (*name != '~')
    {
-   len = (int) strlen(name);
-   if (len > (PATHSET_MAX-1))
+   if (lname > (PATHSET_MAX-1))
       {
-      cpl_msg_error (er_func, "Buffer overflow in filename '%s' - fatal error", name);
+      cpl_msg_error(er_func, 
+                    "Buffer overflow in filename '%s' - fatal error",
+                    name);
       exit (EXIT_FAILURE);
       }
-   (void) strcpy (str, name);
+   return (name);
    }
 
 else
@@ -200,15 +207,27 @@ else
    cpp = getenv ("HOME");
    if (cpp == NULL)
       {
-      cpl_msg_error (er_func, "Env. variable HOME not set, could not replace `~'");
+      cpl_msg_error(er_func, 
+                    "Env. variable HOME not set, could not replace `~'");
       exit (EXIT_FAILURE);
       }
        
-   (void) strcpy (str,cpp);
-   len = (int) strlen(str) + (int) strlen(name);	/* +1 for \0 */
-   if (len > PATHSET_MAX)				/* -1 for (name+1) below */
+   len = (int) strlen(cpp);
+   if (len > (PATHSET_MAX-1))
       {
-      cpl_msg_error (er_func, "Buffer overflow in filename '%s' - fatal error", name);
+      cpl_msg_error (er_func, 
+                     "Buffer overflow in filename '%s' - fatal error",
+                      name);
+      exit (EXIT_FAILURE);
+      }
+   (void) strcpy (str,cpp);
+
+   len += lname;				/* +1 for \0 */
+   if (len > PATHSET_MAX)			/* -1 for (name+1) below */
+      {
+      cpl_msg_error(er_func, 
+                    "Buffer overflow in filename '%s' - fatal error",
+                    name);
       exit (EXIT_FAILURE);
       }
 
@@ -257,6 +276,12 @@ return (str);
  * dot replaced by the current working directory (as an absolute path)
  * using the $PWD environment variable. If not, then the string is 
  * returned unmodified.
+ *
+ * OJO: this routine returns a pointer to a static string,
+ *      if you need the returned filename later on, you must copy it
+ *      to a string of PATHSET_MAX length
+ *      otherwise the returned pointer may point later on to something
+ *      very different   :-(
  */
 /**********************************************************************/
 
@@ -266,7 +291,7 @@ const char *er_fileutils_dot_replace (const char * name)
 static char str[PATHSET_MAX];
 char  *cpp;
 
-int  len;
+int  len, lname;
 
 
 
@@ -275,20 +300,21 @@ ER_TRACE(__func__)
 
 if (name == NULL) return NULL;
 
+lname = (int) strlen(name);
 
 /* if we don't start with a dot (.) just pass the name on */
 
 if (*name != '.') 
    {
-   len = (int) strlen(name);
-   if (len > (PATHSET_MAX-1))
+   if (lname > (PATHSET_MAX-1))
       {
-      cpl_msg_error (er_func, "Buffer overflow in filename '%s' - fatal error", name);
+      cpl_msg_error (er_func,
+                    "Buffer overflow in filename '%s' - fatal error",
+                     name);
       exit (EXIT_FAILURE);
       }
 
-   (void) strcpy (str, name);
-   return (str);
+   return (name);
    }
 
 
@@ -300,7 +326,9 @@ if (cpp != NULL)
    len = (int) strlen(cpp);
    if (len > (PATHSET_MAX-1))
       {
-      cpl_msg_error (er_func, "Buffer overflow in filename '%s' - fatal error", name);
+      cpl_msg_error (er_func, 
+                     "Buffer overflow in filename '%s' - fatal error",
+                      name);
       exit (EXIT_FAILURE);
       }
 
@@ -308,7 +336,7 @@ if (cpp != NULL)
    }
 else
    {
-   cpl_msg_error (er_func, "Env. variable PWD not set - fatal errorn");
+   cpl_msg_error (er_func, "Env. variable PWD not set - fatal error");
    exit (EXIT_FAILURE);
    }
 
@@ -319,23 +347,25 @@ if (*(name + 1) == '.')
    {
    if ((len+2) > (PATHSET_MAX-1))
       {
-      cpl_msg_error (er_func, "Buffer overflow in filename '%s' - fatal error", name);
+      cpl_msg_error (er_func, 
+                     "Buffer overflow in filename '%s' - fatal error", 
+                     name);
       exit (EXIT_FAILURE);
       }
    (void) strcat (str, "/.");
    len += 2;
    }
 
-len = (int) strlen(str) + (int) strlen(name);		/* +1 for \0 */
-if (len > PATHSET_MAX)					/* -1 for (name+1) below */
+len += lname;					/* +1 for \0 */
+if (len > PATHSET_MAX)				/* -1 for (name+1) below */
    {
    cpl_msg_error (er_func, "Buffer overflow in filename '%s'", name);
    cpl_msg_error (er_func, "Fatal error replacing current working "
                   "directory symbol due to buffer overflow");
    exit (EXIT_FAILURE);
    }
-(void) strcat(str, name + 1);		/* Add the rest of the file name */
 
+(void) strcat(str, name + 1);		/* Add the rest of the file name */
 return (str);
 }                            
 /*
@@ -363,13 +393,14 @@ DIR *dp;
 ER_TRACE(__func__)
 
 
-if (directory_name == NULL) return 0;
-
-dp = opendir (er_fileutils_tilde_replace (directory_name));
-if (dp != NULL)
-   {
-   (void) closedir (dp);
-   return 1;			/* that's TRUE */
+if (directory_name != NULL) 
+   {				/* no need to save the pointer, here! */
+   dp = opendir (er_fileutils_tilde_replace (directory_name));
+   if (dp != NULL)
+      {
+      (void) closedir (dp);
+      return 1;			/* that's TRUE */
+      }
    }
 
 return 0;			/* that's FALSE */
@@ -389,43 +420,6 @@ return 0;			/* that's FALSE */
  */
 /**********************************************************************/
 
-int fileutils_file_is_there (const char * file_name)
-
-{
-int file_descriptor;
-
-
-
-ER_TRACE(__func__)
-
-if (file_name != NULL) 
-   {
-   file_descriptor = open (file_name, O_RDONLY);
-   if (file_descriptor > -1)
-      {
-      (void) close (file_descriptor);
-      return 1;
-      }
-   }
-
-return 0;
-
-}                               /* End of fileutils_file_is_there() */
-/*
-^L
-*/
-
-/**********************************************************************/
-/**
- * @brief
- *   Checks whether the given file actually exists
- *
- * @param file_name Filename
- *
- * @returns 1 if file exists, 0 otherwise
- */
-/**********************************************************************/
-
 int fileutils_file_exists (const char * file_name)
 
 {
@@ -439,8 +433,9 @@ const char *resname;
 ER_TRACE(__func__)
 
 if (file_name != NULL) 
-   {
-   resname = er_fileutils_tilde_replace (file_name);
+   {				/* no need to save the pointer, here! */
+   resname = er_fileutils_dot_replace (er_fileutils_tilde_replace (file_name));
+
    file_descriptor = open (resname, O_RDONLY);
    if (file_descriptor > -1)
       {
@@ -449,9 +444,8 @@ if (file_name != NULL)
       }
    }
 
-return 0;
-
-}                               /* End of fileutils_file_exists() */
+return 0;			/* that's FALSE */
+} 
 /*
 ^L
 */
@@ -734,6 +728,8 @@ return 0;
  * returns and the source file is left untouched.
  */
 
+
+
 int fileutils_move (const char *srcpath, const char *dstpath)
 
 {
@@ -767,7 +763,9 @@ if (stat (srcpath, &sb) == -1 || !(sb.st_mode & S_IWUSR))
    (void) unlink (srcpath);
 return 0;
 
-}
+} 
+
+
 /*
 ^L
 */
@@ -847,7 +845,8 @@ if (len > FILEMAX)
 
 /* check if link already there... */
 
-if (fileutils_file_is_there (linkname) == 1) 
+/* if (fileutils_file_is_there (linkname) == 1)  */
+if (fileutils_file_exists (linkname) == 1) 
    {
    cpl_msg_warning (er_func, "the link %s already exists - will be overwritten!",linkname);
    }
@@ -1005,7 +1004,7 @@ return is_fits;
 
 
 
-/*            orig code of Enrique
+/*            orig code of Enrique using CFITSIO functions
 
 int er_fileutils_file_is_fits(const char * self)
 {
