@@ -1,4 +1,4 @@
-/* $Id: er_main.c,v 1.80 2010/11/22 14:05:59 kbanse Exp $
+/* $Id: er_main.c,v 1.87 2012/03/09 17:29:54 kbanse Exp $
  *
  *   This file is part of the ESO Common Pipeline Library
  *   Copyright (C) 2001-2009 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: kbanse $
- * $Date: 2010/11/22 14:05:59 $
- * $Revision: 1.80 $
- * $Name: esorex-3_9_0 $
+ * $Date: 2012/03/09 17:29:54 $
+ * $Revision: 1.87 $
+ * $Name: esorex-3_9_6 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -92,6 +92,7 @@ static void er_init_parameters (cpl_parameterlist * param_list)
 {
 cpl_parameter *p;
 
+
   /*
    * Processing different EsoRex options
    */
@@ -512,8 +513,6 @@ while (p != NULL)
    p = cpl_parameterlist_get_next (param_list);
    }                            /* End of loop through all parameters in the list */
 
-
-
 }                               /* End of er_init_parameters() */
 
 # endif
@@ -534,9 +533,6 @@ static void er_print_header (void)
 
 
 printf ("\n     ***** %s, version %s  *****\n", PACKAGE_NAME, PACKAGE_VERSION);
-
-/* printf ("\n           as of 091217\n"); */
-
 fflush(stdout);
 
 /* 
@@ -587,7 +583,10 @@ int main(int argc, char *argv[])
 const char *cpp, *fn = "EsoRex";
 
 char  plugin_name[FILEMAX+2];
+char  date_str[8];
 char  *conf_file_global=NULL, *conf_file_local=NULL;
+ 
+const char *cdescr;
 
 cpl_parameter *p = NULL;
 cpl_parameterlist *caller_parameter_list = NULL;
@@ -614,6 +613,11 @@ if (getuid() != geteuid())
    exit(EXIT_FAILURE);
    }
 
+/* print version of Esorex, and set date of last modif */
+
+er_print_header ();
+(void) strcpy(date_str,"120311");
+
 
 /* Initialization and setup */
 
@@ -622,8 +626,7 @@ cpl_init (init_flag);                    /* also does cpl_msg_init() stuff */
 cpl_msg_set_domain (PACKAGE);
 cpl_msg_set_time_off ();
 cpl_msg_set_domain_on ();
-
-er_print_header ();
+cpl_msg_set_component_off ();
 
 
 
@@ -697,7 +700,9 @@ else
 (void) strcat(conf_file_global, "/");
 (void) strcat(conf_file_global, GLOBAL_RC_NAME);
 (void) strcat(conf_file_global, GLOBAL_RC_EXTENSION);
-
+/* 
+printf("USE_GLOBAL_ESOREX_RC - conf_file_global = %s\n\n",conf_file_global); 
+*/
 
 /* check if resource file exists, 
    use .rc file from --prefix dir in configuration of Esorex if necessary,
@@ -719,8 +724,8 @@ else
    (void) strcat(new_name, GLOBAL_RC_EXTENSION);
 
    rcfp = fopen(new_name, "r");
-   if (rcfp == NULL)				/* no .rc file in --prefix dir, */
-      {						/* keep original name for compatibility */
+   if (rcfp == NULL)		/* no .rc file in --prefix dir, */
+      {				/* keep original name for compatibility */
       (void) strcpy(conf_file_global,save_name);
       }
    else
@@ -748,34 +753,21 @@ er_init_parameters (caller_parameter_list);
 
 if (argc == 1)		/* entering just Esorex yields the library versions */
    {
-   const char *cdescr;
-
    cdescr = cpl_get_description(CPL_DESCRIPTION_DEFAULT);
    (void) printf("\nLibraries used: %s\n\n",cdescr);
    goto clean_up;
    }
 
-/* if ((argc > 1) && ((int)strcmp(argv[1],"--V") == 0)) */
+cdescr = cpl_get_description(CPL_DESCRIPTION_DEFAULT);
+
 if ((argc == 2) && ((int)strcmp(argv[1],"--V") == 0)) 
    {
-   const char *cdescr;
-
-   (void) printf("           last modif - 101118 \n");
-
-   cdescr = cpl_get_description(CPL_DESCRIPTION_DEFAULT);
+   (void) printf("           as of %s \n",date_str);
    (void) printf("\nLibraries used: %s\n\n",cdescr);
-
    (void) printf("using %s as resource file\n",conf_file_global);
-
-   /* 
-   n = er_fileutils_file_is_fits(argv[2]);
-   printf("er_fileutils_file_is_fits(%s) returned %d\n",argv[2],n);
-   */
-
    goto end_of_it;
    }
 
-(void) printf("\n");
 
 /* Process caller configuration information */
 
@@ -795,7 +787,7 @@ if (e_code == 0)
    {
    e_code = plugin_process_plugin (caller_parameter_list,
                                    plugin_name, set_of_frames_filenames,
-                                   argc, argv);
+                                   date_str,argc, argv);
  
    (void) sprintf(trace_msg,"plugin_process_plugin returned %d",e_code); 
    ER_TRACEX(trace_msg)
