@@ -1,7 +1,7 @@
-/* $Id: er_plugin.c,v 1.97 2010/05/19 08:20:52 kbanse Exp $
+/* $Id: er_plugin.c,v 1.109 2012/05/08 08:10:27 rpalsa Exp $
  *
  *   This file is part of the ESO Common Pipeline Library
- *   Copyright (C) 2001-2009 European Southern Observatory
+ *   Copyright (C) 2001-2012 European Southern Observatory
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -19,10 +19,10 @@
  */
 
 /*
- * $Author: kbanse $
- * $Date: 2010/05/19 08:20:52 $
- * $Revision: 1.97 $
- * $Name: esorex-3_9_0 $
+ * $Author: rpalsa $
+ * $Date: 2012/05/08 08:10:27 $
+ * $Revision: 1.109 $
+ * $Name: esorex-3_9_6 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -62,6 +62,7 @@ static char trace_msg[80];
 static cpl_frameset *holdme;
 
 #define	SIZE_A  2048
+#define	SIZE_AA	2 * SIZE_A
 #define	SIZE_B	128
 #define	SIZE_C	1024
 #define	SIZE_D	1024
@@ -401,15 +402,15 @@ while( fgets (line, (SIZE_C-1), fp))
                if (flag_check_sof_exist)
         	   {
                    cpl_msg_error (er_func,
-                        "The input data file '%s', specified on line %d of the SOF "
-                        "file, '%s', does not exist...", path, line_number, name);
+                        "The input data file '%s', in line %d of the SOF "
+                        "file does not exist...", path, line_number);
                    load_sucesfull = 0;
         	   }
                else
                   {
                   cpl_msg_debug (er_func,
-                       "The input data file '%s', specified on line %d of the SOF "
-                       "file, '%s', does not exist...", path, line_number, name);
+                       "The input data file '%s', in line %d of the SOF "
+                       "file, does not exist...", path, line_number);
                   }
                }
             }
@@ -577,213 +578,337 @@ return 0;
 
 /* int move_products (char * plugin_name, */
 int move_products (cpl_frameset * frameset,
-             char * output_directory,
-             char * output_prefix,
-             char * link_directory,
-             int flag_noprefix, int flag_nolink, int flag_readonly)
+                   char * output_directory,
+                   char * output_prefix,
+                   char * link_directory,
+                   int flag_noprefix, int flag_nolink, int flag_readonly)
 
 {
-cpl_frame *frame = NULL;		/* The current frame being processed */
+    cpl_frame *frame = NULL;       /* The current frame being processed */
 
-int filenum = 0;			/* Number of the output product */
-int ii, move_err;		
+    int filenum = 0;			   /* Number of the output product */
+    int ii, move_err;
+    int target_path_is_cwd = 0;    /* assume that output path and CWD differ */
 
-const char *input_name = NULL;          /* Name of the input file */
-const char *output_dir = NULL;          /* Name of the output directory */
-const char *link_dir = NULL;            /* Name of the symbolic link dir. */
-const char *cpp;
+    const char *fileutils_pntr;
+    const char *cpp;
+    const char *input_name = NULL;       /* Name of the input file */
 
-char *output_name = NULL;               /* Name of the output file */
-char *suffix = NULL;                    /* The file suffix */
-char current_dir[FILEMAX];         /* Name of current (work) directory */
+    char *output_name = NULL;            /* Name of the output file */
+    char *suffix = NULL;                 /* The file suffix */
+    char current_dir[FILEMAX];           /* Name of current (work) directory */
+    char  command_str[SIZE_AA];
+    char link_dir[PATHSET_MAX];          /* Name of the symbolic link dir. */
+    char output_dir[PATHSET_MAX];        /* Name of the output dir. */
+
+    struct stat sb_cdir;
+    struct stat sb_tdir;
+
+    cpl_msg_severity msg_level;
 
 
-ER_TRACE(__func__);
+    ER_TRACE(__func__);
 
 
-/* allocate space */
+    /* allocate space */
 
-output_name = (char *) cpl_malloc((size_t) SIZE_A);
+    output_name = (char *) cpl_malloc((size_t) SIZE_A);
 
-current_dir[0] = '\0';
-
- 
-/* Start by getting the "cleaned" output directory name */
-
-output_dir = 
-   er_fileutils_dot_replace (er_fileutils_tilde_replace (output_directory));
-link_dir = er_fileutils_tilde_replace (link_directory);
-
-if (link_dir == NULL)
-   {
-   cpl_msg_warning (er_func, "missing directory for symbolic link ignored...");
-   }
-
-/* Determine the name of the current directory */
-
-(void) getcwd (current_dir, (size_t) FILEMAX);
-
-if (ER_TRACEFLAG)
-   {
-   (void) sprintf(trace_msg, "output_directory = %s>",output_directory);
-   ER_TRACEX(trace_msg)
-   (void) sprintf(trace_msg, "output_dir = %s>",output_dir);
-   ER_TRACEX(trace_msg)
-   (void) sprintf(trace_msg, "current_dir = %s>",current_dir);
-   ER_TRACEX(trace_msg)
-   (void) sprintf(trace_msg, "link_dir = %s>",link_dir);
-   ER_TRACEX(trace_msg)
-   }
-
-ii = upda_products(frameset);		/* MD5 errors ignored at the moment... */
-holdme = frameset;
+    current_dir[0] = '\0';
 
 
-/* Loop through the frameset */
+    /* Start by getting the "cleaned" output directory name */
 
-move_err = 0;				/* init possible error from moving files */
-frame = cpl_frameset_get_first (frameset);
-while (frame != NULL)
-   {
-   if (cpl_frame_get_group (frame) == CPL_FRAME_GROUP_PRODUCT)
-      {
+    fileutils_pntr =
+            er_fileutils_dot_replace (er_fileutils_tilde_replace (output_directory));
+    if (fileutils_pntr == NULL)
+    {
+        cpl_msg_warning (er_func, "missing output directory ...");
+    }
+    (void) strcpy (output_dir, fileutils_pntr);
 
-      /* Get the filename as specified by the frame */
-      input_name = cpl_frame_get_filename (frame);
+    fileutils_pntr = er_fileutils_tilde_replace (link_directory);
+    if (fileutils_pntr == NULL)
+    {
+        cpl_msg_warning (er_func, "missing directory for symbolic link ignored...");
+    }
+    (void) strcpy (link_dir, fileutils_pntr);
 
-      /* Temporarily remove the suffix (find the first '.' and replace it) */
-      suffix = strrchr (input_name, '.');
-      if (suffix) *suffix = '\0';
 
-      /* Generate the output filename (keep original if this is suppressed) */
-      if (flag_noprefix != 0)
-         {
-         (void) strcpy (output_name, input_name);
-         }
-      else
-         {
-         (void) sprintf (output_name, "%s_%04d", output_prefix, filenum);
-         }
+    /* Determine the name of the current directory */
 
-      /* Now restore the suffix (note difference between suffix and *suffix) */
-      if (suffix) *suffix = '.';
-      if (suffix) (void) strcat (output_name, suffix);
+    (void) getcwd (current_dir, (size_t) FILEMAX);
 
-      /* Check if any move-command is required */
+    if (ER_TRACEFLAG)
+    {
+        (void) sprintf(trace_msg, "output_directory = %s>",output_directory);
+        ER_TRACEX(trace_msg)
+        (void) sprintf(trace_msg, "output_dir = %s>",output_dir);
+        ER_TRACEX(trace_msg)
+        (void) sprintf(trace_msg, "current_dir = %s>",current_dir);
+        ER_TRACEX(trace_msg)
+        (void) sprintf(trace_msg, "link_dir = %s>",link_dir);
+        ER_TRACEX(trace_msg)
+    }
 
-         cpp = cpl_frame_get_filename (frame);
-         if (cpp == NULL)
+    // FIXME: This is a hot fix to avoid a failing move of the product
+    //        files in case the output directory is identical to the
+    //        current directory but referenced through a link so that
+    //        the two path names are different.
+    //        Finally this section should use the file utilities function,
+    //        which need to be reviewed first.
+
+    if ((stat(current_dir, &sb_cdir) != 0) || (stat(output_dir, &sb_tdir) != 0))
+    {
+        cpl_msg_error(er_func, "cannot get directory status information");
+        move_err = 65;
+        goto report_products;
+    }
+    else
+    {
+
+        /*
+         * Check whether the current working directory and the final
+         * location of the product files are identical, i.e. they are
+         * located on the same device and they have the the same inode.
+         */
+
+        if ((sb_cdir.st_dev == sb_tdir.st_dev) &&
+                (sb_cdir.st_ino == sb_tdir.st_ino))
+        {
+            target_path_is_cwd = 1;
+        }
+
+    }
+
+    ii = upda_products(frameset);     /* MD5 errors ignored at the moment... */
+    holdme = frameset;
+
+
+    /* Loop through the frameset */
+
+    move_err = 0;				/* init possible error from moving files */
+    frame = cpl_frameset_get_first (frameset);
+    while (frame != NULL)
+    {
+        if (cpl_frame_get_group (frame) == CPL_FRAME_GROUP_PRODUCT)
+        {
+
+            /*
+             *  Get the filename as specified by the frame
+             */
+
+            input_name = cpl_frame_get_filename (frame);
+
+            /*
+             * Temporarily remove the suffix (find the first '.' and
+             * replace it)
+             */
+
+            suffix = strrchr (input_name, '.');
+            if (suffix) *suffix = '\0';
+
+            /*
+             *  Generate the output filename (keep original if this
+             * is suppressed)
+             */
+
+            if (flag_noprefix != 0)
             {
-            cpl_msg_error (er_func, "cpl_frame_get_filename failed");
-            move_err = 63;
-            goto report_products;
+                (void) strcpy (output_name, input_name);
+            }
+            else
+            {
+                (void) sprintf (output_name, "%s_%04d", output_prefix,
+                                filenum);
             }
 
-      if (((strcmp (current_dir, output_dir) == 0) ||
-           (strcmp (".", output_directory) == 0)) &&			/* same dir */
-          (strcmp (cpp, output_name) == 0))	/* same file */
-         {
-         /* No move. Report that the product has been created */
-         cpl_msg_info (er_func, "Created product %s (in place)", output_name);
-         }
+            /*
+             * Now restore the suffix (note difference between
+             *  suffix and *suffix)
+             */
 
-      else
-         {						/* Create the full path */
-         char  tmp[SIZE_A];
+            if (suffix) *suffix = '.';
+            if (suffix) (void) strcat (output_name, suffix);
 
-         if (ER_TRACEFLAG)
+            /*
+             *  Check if any move-command is required
+             */
+
+            cpp = cpl_frame_get_filename (frame);
+            if (cpp == NULL)
             {
-            (void) sprintf(trace_msg, "input name = %s>",cpp);
-            ER_TRACEX(trace_msg)
-            (void) sprintf(trace_msg, "output_name = %s>",output_name);
-            ER_TRACEX(trace_msg)
+                cpl_msg_error (er_func, "cpl_frame_get_filename failed");
+                move_err = 63;
+                goto report_products;
             }
 
-         if (output_name[0] == '\0')
-            (void) strcpy(tmp,"dummy_output");
-         else
-            (void) strcpy(tmp,output_name);
 
-         (void) strcpy (output_name, output_dir);
-         ii = (int) strlen(output_name) - 1;
-         if (output_name[ii] != '/') output_name[++ii] = '/';
-         (void) strcpy (&output_name[ii+1],tmp);
+            /*
+             * Move product files only if the current directory and product
+             * target path are different, or if the local product name and
+             * the final product name are different.
+             */
 
-         if (ER_TRACEFLAG)
+            if (target_path_is_cwd && (strcmp(cpp, output_name) == 0))
             {
-            (void) sprintf(trace_msg, "input = %s",cpp);
-            ER_TRACEX(trace_msg)
-            (void) sprintf(trace_msg, "final output_name = %s",output_name);
-            ER_TRACEX(trace_msg)
+
+                /*
+                 *  No move. Report that the product has been created
+                 */
+
+                cpl_msg_info (er_func, "Created product %s (in place)",
+                              output_name);
+
             }
-             
-/*  
+            else
+            {
+
+                /*
+                 *  Create the full path
+                 */
+
+                char  tmp[SIZE_A];
+
+                if (ER_TRACEFLAG)
+                {
+                    (void) sprintf(trace_msg, "input name = %s>",cpp);
+                    ER_TRACEX(trace_msg)
+                    (void) sprintf(trace_msg, "output_name = %s>",output_name);
+                    ER_TRACEX(trace_msg)
+                }
+
+                if (output_name[0] == '\0')
+                    (void) strcpy(tmp,"dummy_output");
+                else
+                    (void) strcpy(tmp,output_name);
+
+                (void) strcpy (output_name, output_dir);
+                ii = (int) strlen(output_name) - 1;
+                if (output_name[ii] != '/') output_name[++ii] = '/';
+                (void) strcpy (&output_name[ii+1],tmp);
+
+                if (ER_TRACEFLAG)
+                {
+                    (void) sprintf(trace_msg, "input = %s",cpp);
+                    ER_TRACEX(trace_msg)
+                    (void) sprintf(trace_msg, "final output_name = %s",output_name);
+                    ER_TRACEX(trace_msg)
+                }
+
+                /*
          rename() has problems:
 	 a) it happily overwrites read-only files...
 	 b) it does not work across different partitions (that's documented)
 	 so we cannot simply do:
 
          ret_val = rename (cpp, output_name);
-*/
+.......
 
-         ii = fileutils_move (cpp, output_name);   
-         if (ii != 0)
-            {				/* also that failed - so, no way... */
-            if (ii == 99)
-               {			/* different name, but same inode ... */
-               cpl_msg_info (er_func, "Created product %s (in place)", output_name);
+         if (flag_outcopy == 0)		/. we just move the products ./
+            {
+            char  command_str[SIZE_AA];
+            cpl_msg_severity msg_level;
+
+            strcpy(command_str,"mv ");
+            strcat(command_str,cpp);
+            strcat(command_str," ");
+            strcat(command_str,output_name);
+
+            msg_level = cpl_msg_get_level(); /. test, if we are in debug mode ./
+            if (msg_level == CPL_MSG_DEBUG)
+               {				/. if debug level set, ./
+               printf("move command:\n%s\n",command_str);
                }
-            else
+
+            ii = system(command_str);
+            if (ii != 0) 
                {
-               cpl_msg_error (er_func, "Unable to move product file to final "
-                        "output location (%s)", output_name);
-               move_err = 61;
-               (void) sprintf(trace_msg, "fileutils_move returned: %d",ii);
+               cpl_msg_error(er_func, "Error moving product file '%s'", cpp);
+               move_err = 64;
+               (void) sprintf(trace_msg, "move command returned: %d",ii);
                ER_TRACEX(trace_msg)
                goto report_products;
                }
             }
          else
             {
-            /* Update the name in the sof */ 
-            cpl_frame_set_filename(frame, output_name);
-            
-            cpl_msg_info (er_func, "Created product %s", output_name);
+            ii = fileutils_move (cpp, output_name);   
+            if (ii != 0)
+               {			/. also that failed - so, no way... ./
+               if (ii == 99)
+                  {			/. different name, but same inode ... ./
+                  cpl_msg_info (er_func, "Created product %s (in place)", output_name);
+                  }
+               else
+                  {
+                  cpl_msg_error (er_func,"Unable to move product file to final "
+                           "output location (%s)", output_name);
+                  move_err = 61;
+                  (void) sprintf(trace_msg, "fileutils_move returned: %d",ii);
+                  ER_TRACEX(trace_msg)
+                  goto report_products;
+                  }
+               }
             }
-         }
+                 */
 
-      /* If required, generate the symbolic links for the archive system */
-      if (flag_nolink == 0) 
-         {
-         if (er_fileutils_link (link_dir, output_name) != EXIT_SUCCESS)
+                strcpy(command_str,"mv ");
+                strcat(command_str,cpp);
+                strcat(command_str," ");
+                strcat(command_str,output_name);
+                msg_level = cpl_msg_get_level();       /* if in in debug mode */
+                if (msg_level == CPL_MSG_DEBUG)
+                {                           /* print the command string */
+                    printf("move command:\n%s\n",command_str);
+                }
+                ii = system(command_str);
+                if (ii != 0)
+                {                   /* now, product file should be there... */
+                    cpl_msg_error (er_func,
+                                   "Unable to move product file to final output path (%s)",
+                                   output_name);
+                    move_err = 61;
+                    goto report_products;
+                }
+
+                /* Update the name in the sof */
+                cpl_frame_set_filename(frame, output_name);
+                cpl_msg_info (er_func, "Created product %s", output_name);
+            }
+
+            /* If required, generate the symbolic links for the archive system */
+            if (flag_nolink == 0)
             {
-            cpl_msg_error (er_func,
-                           "Unable to create symbolic link for %s in "
-                           "directory %s", output_name, link_dir);
-            move_err = 62;
-            goto report_products;
+                if (er_fileutils_link (link_dir, output_name) != EXIT_SUCCESS)
+                {
+                    cpl_msg_error (er_func,
+                                   "Unable to create symbolic link for %s in "
+                                   "directory %s", output_name, link_dir);
+                    move_err = 62;
+                    goto report_products;
+                }
             }
-         }
 
-      /* If required, lock the file permissions to read-only */
-      if (flag_readonly != 0) chmod (output_name, 0444);
+            /* If required, lock the file permissions to read-only */
+            if (flag_readonly != 0) chmod (output_name, 0444);
 
-      filenum++;		/* Increment the product number */
-      }                   
+            filenum++;		/* Increment the product number */
+        }
 
-   /* Get the next frame in the list */
+        /* Get the next frame in the list */
 
-   frame = cpl_frameset_get_next (frameset);
-   }                            /* End of (while) loop through all frames in frameset */
+        frame = cpl_frameset_get_next (frameset);
+    }                            /* End of (while) loop through all frames in frameset */
 
 
-/* Report how many products were created (getting the plural right!) */
+    /* Report how many products were created (getting the plural right!) */
 
-report_products:
-cpl_msg_info (er_func, "%d product%s created", filenum, (filenum == 1) ? "" : "s");
-cpl_free (output_name);				/* free the allocated memory */
+    report_products:
+    cpl_msg_info (er_func, "%d product%s created", filenum, (filenum == 1) ? "" : "s");
+    cpl_free (output_name);				/* free the allocated memory */
 
-return move_err;
+    return move_err;
 }                            
 /*
 
@@ -807,9 +932,10 @@ return move_err;
  */
 /**********************************************************************/
 
-int plugin_process_plugin (cpl_parameterlist * caller_parameters,
-            char * plugin_name,
-            er_stringarray_t * sof_filename_list,
+int plugin_process_plugin (cpl_parameterlist *caller_parameters,
+            char *plugin_name,
+            er_stringarray_t * sof_filename_list, 
+            char *date_str,
             int argc, char * argv[])
 
 {
@@ -897,10 +1023,14 @@ log_file_name_full =
 
 /* CPL messaging should already be active, but reset the defaults */
 
+/*
 cpl_msg_set_domain (PACKAGE);
 cpl_msg_set_time_off ();
 cpl_msg_set_domain_on ();
 cpl_msg_set_component_off ();
+cpl_msg_info(er_func,"logfile name = %s\n",log_file_name_full);
+*/
+
 log_err = cpl_msg_set_log_level (msg_sev_logfile);
 if (log_err != CPL_ERROR_NONE)
    {
@@ -914,7 +1044,10 @@ if (log_err != CPL_ERROR_NONE)
 cpl_msg_set_level (CPL_MSG_OFF);
 cdescr = cpl_get_description(CPL_DESCRIPTION_DEFAULT);
 
-(void) cpl_msg_info(er_func,"Libraries used: %s\n",cdescr);
+(void) cpl_msg_info(er_func,"Esorex, version %s, as of %s",PACKAGE_VERSION,date_str);
+/*
+*/
+(void) cpl_msg_info(er_func,"using the libraries: %s\n",cdescr);
 cpl_msg_set_level (msg_sev_terminal);
 
 
@@ -973,12 +1106,6 @@ if ((val_recipe_dirs != NULL) && (sz > 0))
 /* ------------------------------------- */
 
 next_step:
-
-/*
-printf("none = %d, recipe = %d, recipe_v2 = %d\n",CPL_PLUGIN_TYPE_NONE,CPL_PLUGIN_TYPE_RECIPE,CPL_PLUGIN_TYPE_RECIPE_V2);
-*/
-
-
 uil = cpl_plugin_get_type(tplugin);
 
 /*  initialize the plugin/recipe structure: trecipe->parameters is handled by plugin  */
@@ -1380,7 +1507,8 @@ if (flag_plugin_found != 0)			/*  = TRUE */
 
       e_code = move_products (trecipe->frames, (char *)val_output_dir,
                    (char *)val_output_mask,
-                   (char *)val_link_dir, flag_noprefix, flag_nolink, flag_readonly);
+                   (char *)val_link_dir, flag_noprefix, 
+		   flag_nolink, flag_readonly);
 
       if (e_code != 0) cpl_msg_error (er_func,
                         "An error occurred while trying to move the output products");
@@ -1458,7 +1586,7 @@ if ((flag_time_plugin != 0) && (flag_plugin_found != 0))
    plugin_time_end = cpl_test_get_walltime();
    if (plugin_time_end > plugin_time_start)
       {
-      plugin_time_end -= plugin_time_start;		/* reuse for time difference */
+      plugin_time_end -= plugin_time_start;	/* reuse for time difference */
       }
    else
       {
@@ -1475,7 +1603,7 @@ if ((flag_time_plugin != 0) && (flag_plugin_found != 0))
          input_name = cpl_frame_get_filename (frame);  
          if (add_size (input_name,&fsz) != 0)		/* accumulate size (in bytes) of raw input frames */
             {
-            cpl_msg_warning("could not get size of %s\n",input_name);
+            cpl_msg_warning(er_func,"could not get size of %s\n",input_name);
             }
          else
             {
@@ -1522,41 +1650,39 @@ if ((flag_time_plugin != 0) && (flag_plugin_found != 0))
 /* Terminate CPL messaging */
 
 cleanup:
+
+/*
+cpl_msg_set_component_off ();
+cpl_msg_set_time_off ();
+cpl_msg_set_domain_off ();
+*/
+
 /* currently cpl_msg_stop_log () always returns CPL_ERROR_NONE ... 080613 */
 if (cpl_msg_stop_log () != CPL_ERROR_NONE)
    cpl_msg_error (er_func, "An error was encountered while closing the logfile");
    
-if (flag_plugin_found != 0)
+else
    {						/*  Move log file...  */
    char  tmpbuf[FILEMAX+12];
 
    (void) getcwd (tmpbuf, (size_t) FILEMAX);
    (void) strcat (tmpbuf, "/.logfile");
 
-   if (fileutils_file_is_there(tmpbuf) != 0)
+   if (fileutils_file_exists(tmpbuf) != 0)
       {						/* .logfile exists ... */
-      char  dstbuf[FILEMAX+2];
-
-      (void) strcpy (dstbuf, val_log_dir);
-      (void) strcat (dstbuf, "/");
-      (void) strcat (dstbuf, val_log_file);
-      n = fileutils_copy(tmpbuf,dstbuf);  /* copy .logfile to "real" log file */
+      n = fileutils_copy(tmpbuf,log_file_name_full);	/* copy .logfile to "real" log file */
       if (n < 0)
          {
          (void) printf("we could not copy .logfile to %s (err-code = %d)\n",
-                       dstbuf,n);
+                       log_file_name_full,n);
          }
       else
-         {
-         if (n == 0) (void) unlink(tmpbuf);	/* get rid of .logfile */
+         {						/* get rid of .logfile  */
+         if (n == 0) (void) unlink(tmpbuf);	 
          }
       }
    }
 
-cpl_msg_set_component_off ();
-cpl_msg_set_time_off ();
-cpl_msg_set_domain_off ();
-
 
 /* free memory again */
 
