@@ -1,4 +1,4 @@
-/* $Id: er_plugin.h,v 1.14 2009/12/11 16:35:22 cgarcia Exp $
+/* $Id: er_plugin.h,v 1.17 2012/02/23 13:48:05 kbanse Exp $
  *
  *   This file is part of the ESO Common Pipeline Library
  *   Copyright (C) 2001-2008 European Southern Observatory
@@ -19,10 +19,10 @@
  */
 
 /*
- * $Author: cgarcia $
- * $Date: 2009/12/11 16:35:22 $
- * $Revision: 1.14 $
- * $Name: esorex-3_9_0 $
+ * $Author: kbanse $
+ * $Date: 2012/02/23 13:48:05 $
+ * $Revision: 1.17 $
+ * $Name: esorex-3_9_6 $
  */
 
 
@@ -45,6 +45,7 @@ extern int plugin_process_plugin
   cpl_parameterlist *,
   char *,
   er_stringarray_t *,
+  char *,
   int, 
   char **
 );
