@@ -1,4 +1,4 @@
-/* $Id: er_pluginlist.c,v 1.38 2010/07/09 08:33:06 kbanse Exp $
+/* $Id: er_pluginlist.c,v 1.39 2012/02/23 13:46:14 kbanse Exp $
  *
  *   This file is part of the ESO Common Pipeline Library
  *   Copyright (C) 2001-2006 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: kbanse $
- * $Date: 2010/07/09 08:33:06 $
- * $Revision: 1.38 $
- * $Name: esorex-3_9_0 $
+ * $Date: 2012/02/23 13:46:14 $
+ * $Revision: 1.39 $
+ * $Name: esorex-3_9_6 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -428,11 +428,9 @@ lt_dlhandle module = NULL;
 cpl_pluginlist *pl_list = NULL;
 cpl_plugin *pl = NULL;
 
-int i = 0, e_code = 0;
-int f_report_error = 1;
-int num_copies = 0;
+int num_copies = 0, i = 0;
 int num_best = 0;
-int  m;
+int  m, e_code, f_report_error;
 
 int (*get_plugin_list) (cpl_pluginlist *) = NULL;
 
@@ -750,7 +748,7 @@ cpl_plugin *tplugin = NULL;
 
 lt_dlhandle module = NULL;
 
-int m, e_code = 0, i = 0;
+int m, e_code, i = 0;
 int (*get_plugin_list) (cpl_pluginlist *) = NULL;
 
 
@@ -867,7 +865,7 @@ lt_dlhandle module = NULL;
 
 int (*get_plugin_list) (cpl_pluginlist *) = NULL;
 
-int m, e_code = 0, i = 0;
+int m, e_code, i = 0;
 int num_plugins = 0;
 
 
